@@ -51,7 +51,12 @@ class SelectStatement:
             statement_str += " {}".format(transform(self.expressions[0]))
         else:
             statement_str += "\n {}".format(
-                ",\n ".join(map(transform, self.expressions))
+                ",\n ".join(
+                    map(
+                        lambda s: s.replace("\n", "\n "),
+                        map(transform, self.expressions),
+                    )
+                )
             )
 
         if self.from_statement:
@@ -423,7 +428,41 @@ class FunctionCall(Expression):
         )
 
 
+class AnalyticsClause(Expression):
+    def __init__(self, function, partition_by, order_by, frame_clause):
+        self.function = function
+        self.partition_by = partition_by
+        self.order_by = order_by
+        self.frame_clause = frame_clause
+
+    def __str__(self):
+        analytics_str = "{} OVER (".format(transform(self.function))
+        if self.partition_by:
+            analytics_str += "\n PARTITION BY"
+            if len(self.partition_by) > 1:
+                analytics_str += "\n  " + ",\n  ".join(
+                    map(transform, self.partition_by)
+                )
+            else:
+                analytics_str += " " + transform(self.partition_by[0])
+        if self.order_by:
+            analytics_str += "\n ORDER BY{}".format(
+                transform(self.order_by).replace("\n", "\n ")
+            )
+        # TODO: frame_clause
+
+        analytics_str += "\n)" if "\n" in analytics_str else ")"
+        return analytics_str
+
+
 class Column(Expression):
+    KEYWORDS = ("_table_suffix",)
+
+    def __str__(self):
+        if self.value in self.KEYWORDS:
+            return self.value.upper()
+        return str(self.value)
+
     def validate(self, known_fields):
         errors = super().validate(known_fields)
         if self.value not in known_fields and "*" not in known_fields:
@@ -506,7 +545,7 @@ class Parenthesis(Expression):
         self.args = args
 
     def __str__(self):
-        return "({})".format(", ".join(map(str, self.args)))
+        return "({})".format(", ".join(map(transform, self.args)))
 
     def __repr__(self):
         return "<Parenthesis: {}>".format(", ".join(map(repr, self.args)))
@@ -601,6 +640,62 @@ class Table(Expression):
     pass
 
 
+class Join(Expression):
+    VALUES = ("join", "inner", "left", "right", "full", "cross")
+
+    def __init__(self, join_type, left_from, right_from, on, using):
+        self.join_type = join_type
+        self.left_from = left_from
+        self.right_from = right_from
+        self.on = on
+        self.using = using
+
+    def __str__(self):
+        right_from = self.right_from
+        num_parenthesis = 0
+        while isinstance(right_from, Parenthesis):
+            right_from = right_from.args[0]
+            num_parenthesis += 1
+
+        right_element = "\n" + transform(right_from)
+        right_element = right_element.replace("\n", "\n ")
+        if num_parenthesis:
+            right_element = " {}{}\n{}".format(
+                "(" * num_parenthesis, right_element, ")" * num_parenthesis,
+            )
+
+        left_from = self.left_from
+        num_parenthesis = 0
+        while isinstance(left_from, Parenthesis):
+            left_from = left_from.args[0]
+            num_parenthesis += 1
+
+        left_element = transform(left_from)
+        if num_parenthesis:
+            left_element = "\n" + left_element
+            left_element = left_element.replace("\n", "\n ")
+            left_element = "{}{}\n{}".format(
+                "(" * num_parenthesis, left_element, ")" * num_parenthesis,
+            )
+
+        join_str = "{}\n{}{}\n".format(
+            left_element, self.join_type.upper(), right_element,
+        )
+        if self.on:
+            join_str += "ON {}".format(transform(self.on))
+        elif self.using:
+            join_str += "USING {}".format(transform(self.using))
+        return join_str
+
+
+class OnClause(Expression):
+    pass
+
+
+class UsingClause(Expression):
+    pass
+
+
 class Condition(Expression):
     PREDICATES = ("=", ">", "<", "<=", ">=", "is", "like", "<>")
 
@@ -669,3 +764,17 @@ class BooleanCondition(Expression):
     @property
     def return_type(self):
         return bool
+
+
+class ExceptClause(Expression):
+    def __init__(self, expression, args):
+        super().__init__(expression)
+        self.args = args
+
+    def __str__(self):
+        except_str = "{} EXCEPT (".format(transform(self.value))
+        if len(self.args) > 1:
+            except_str += "\n {}\n".format(",\n ".join(map(transform, self.args)))
+        elif except_str:
+            except_str += transform(self.args[0])
+        return except_str + ")"
